@@ -1,5 +1,5 @@
 /* Sniff out modules from ELF headers visible in memory segments.
-   Copyright (C) 2008, 2009 Red Hat, Inc.
+   Copyright (C) 2008-2010 Red Hat, Inc.
    This file is part of Red Hat elfutils.
 
    Red Hat elfutils is free software; you can redistribute it and/or modify
@@ -73,19 +73,28 @@
 #endif
 
 
-/* Return user segment index closest to ADDR but not above it.  */
+/* Return user segment index closest to ADDR but not above it.
+   If NEXT, return the closest to ADDR but not below it.  */
 static int
-addr_segndx (Dwfl *dwfl, size_t segment, GElf_Addr addr)
+addr_segndx (Dwfl *dwfl, size_t segment, GElf_Addr addr, bool next)
 {
-  int ndx = dwfl->lookup_segndx[segment];
+  int ndx = -1;
   do
     {
       if (dwfl->lookup_segndx[segment] >= 0)
 	ndx = dwfl->lookup_segndx[segment];
-      ++segment;
+      if (++segment >= dwfl->lookup_elts - 1)
+	return next ? ndx + 1 : ndx;
+    }
+  while (dwfl->lookup_addr[segment] < addr);
+
+  if (next)
+    {
+      while (dwfl->lookup_segndx[segment] < 0)
+	if (++segment >= dwfl->lookup_elts - 1)
+	  return ndx + 1;
+      ndx = dwfl->lookup_segndx[segment];
     }
-  while (segment < dwfl->lookup_elts - 1
-	 && dwfl->lookup_addr[segment] < addr);
 
   return ndx;
 }
@@ -148,7 +157,7 @@ dwfl_segment_report_module (Dwfl *dwfl, int ndx, const char *name,
       {
 	*data = NULL;
 	*data_size = filesz;
-	return segment_read (addr_segndx (dwfl, segment, vaddr),
+	return segment_read (addr_segndx (dwfl, segment, vaddr, false),
 			     data, data_size, vaddr, filesz);
       }
 
@@ -437,10 +446,8 @@ dwfl_segment_report_module (Dwfl *dwfl, int ndx, const char *name,
   dyn_vaddr += bias;
 
   /* Our return value now says to skip the segments contained
-     within the module.
-     XXX handle gaps
-  */
-  ndx = addr_segndx (dwfl, segment, module_end);
+     within the module.  */
+  ndx = addr_segndx (dwfl, segment, module_end, true);
 
   /* Examine its .dynamic section to get more interesting details.
      If it has DT_SONAME, we'll use that as the module name.
@@ -519,13 +526,28 @@ dwfl_segment_report_module (Dwfl *dwfl, int ndx, const char *name,
   size_t soname_size = 0;
   if (dynstrsz != 0 && dynstr_vaddr != 0)
     {
-      /* We know the bounds of the .dynstr section.  */
-      dynstr_vaddr += bias;
+      /* We know the bounds of the .dynstr section.
+
+	 The DYNSTR_VADDR pointer comes from the .dynamic section
+	 (DT_STRTAB, detected above).  Ordinarily the dynamic linker
+	 will have adjusted this pointer in place so it's now an
+	 absolute address.  But sometimes .dynamic is read-only (in
+	 vDSOs and odd architectures), and sometimes the adjustment
+	 just hasn't happened yet in the memory image we looked at.
+	 So treat DYNSTR_VADDR as an absolute address if it falls
+	 within the module bounds, or try applying the phdr bias
+	 when that adjusts it to fall within the module bounds.  */
+
+      if ((dynstr_vaddr < module_start || dynstr_vaddr >= module_end)
+	  && dynstr_vaddr + bias >= module_start
+	  && dynstr_vaddr + bias < module_end)
+	dynstr_vaddr += bias;
+
       if (unlikely (dynstr_vaddr + dynstrsz > module_end))
 	dynstrsz = 0;
 
       /* Try to get the DT_SONAME string.  */
-      if (soname_stroff != 0 && soname_stroff < dynstrsz - 1
+      if (soname_stroff != 0 && soname_stroff + 1 < dynstrsz
 	  && ! read_portion (&soname, &soname_size,
 			     dynstr_vaddr + soname_stroff, 0))
 	name = soname;
@@ -584,7 +606,7 @@ dwfl_segment_report_module (Dwfl *dwfl, int ndx, const char *name,
       {
 	void *into = contents + offset;
 	size_t read_size = size;
-	(void) segment_read (addr_segndx (dwfl, segment, vaddr),
+	(void) segment_read (addr_segndx (dwfl, segment, vaddr, false),
 			     &into, &read_size, vaddr, size);
       }
 
